function varargout = NFMieCoated( diameter, nindex, lambda, method, varargin )

% NFMIECOATED  Calculates the absorption, scattering and extinction spectra 
% or the field distribution for a general core-multishell particle
% The calculation is made according to Mie theory as presented in
% "Principles of Surface Enhanced Raman Scattering" by LeRu and Etchegoin
%
% Method 1 computes the spectra of the absorption, scattering and
% extinction cross sections
% Calling example for method 1
% [ abs, sca, ext ] = NFMieCoated( 60, [ 0.15 + 1i*3.5, 1.33 ], 800, 1 );
%
% Methods 2 and 3 compute the magnitude of the complex field on a xz plane
% at y=0 and at the surface of the particle respectively
% Calling example for methods 2 or 3
% [ ETot ] = NFMieCoated( 60, [ 0.15 + 1i*3.5, 1.33 ], 800, 2 );
%
% Method 4 computes the magnitude of the complex field in the far field (as
% a function of theta and phi). The dependence on r is not included, so the
% field should be normalized because the absolute magnitude is not
% meaningful here.
% Calling example for method 4
% [ EscaTot ] = NFMieCoated( 60, [ 0.15 + 1i*3.5, 1.33 ], 800, 4 );
%
% Explanation of the inputs
% diameter  =>	diameter of the different shells of the particle, in nm
%               (enter as a vector [d1, d2, d3, etc. ])
% nindex    =>	complex refractive indexes of the different shells of the
%               particle, (enter as a matrix [n1, n2, n3, etc., nmedium ]
%               where n1, n2, etc. are column vectors with the same number
%               of elements as lambda and nmedium contains the wavelength-
%               dependent refractive index of the medium)
%               (nindex must have one more column than the number of
%               elements of diameter)
% lambda    =>	wavelength, in nm (if lambda has more than one element, it
%               must be a column vector) 
% method    =>  1 for Cext Csca & Cabs calculation,
%               2 for field on a xz plane and
%               3 for field on a thin shell around the particle

% Author: David Rioux
% Version number: 025
% Date modified: 2016-05-12
% Note: Corrected and simplified calculation of incident field.

% Default values for undefined inputs
nmax=20;
maxx=diameter(end);
dx=maxx/100;
maxz=diameter(end);
dz=maxz/100;
shell_rad=diameter(end)/2;
dtheta=pi/60;
dphi=pi/60;
separaten=0;

% Sparsing inputs
if mod(length(varargin),2)>0
    error ('Unexpected number of input arguments')
elseif ~isempty(varargin)
    for inputnumber=1:2:length(varargin)-1
        if ischar(varargin{inputnumber})==0
            error('Unexpected input argument')
        elseif strcmp(varargin{inputnumber},'nmax')
            nmax=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'maxx')
            maxx=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'dx')
            dx=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'maxz')
            maxz=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'dz')
            dz=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'shellradius')
            shell_rad=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'dtheta')
            dtheta=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'dphi')
            dphi=varargin{inputnumber+1};
        elseif strcmp(varargin{inputnumber},'separaten')
            separaten=varargin{inputnumber+1};
        end
    end
end


diameter(1,2:end+1) = diameter(1,1:end);
diameter(1) = 0;
diameter(end+1) = inf;

VolNumber = size(nindex,2);

for volume = 1:VolNumber
    k(:,volume) = nindex(:,volume).*(2*pi./lambda);
    x(:,volume) = k(:,volume)*(diameter(volume)/2);
    if volume == 1
        s(:,volume) = zeros(size(lambda));
    else
        s(:,volume) = k(:,volume-1)./k(:,volume);
    end
    sx(:,volume) = s(:,volume).*x(:,volume);
end

% Preallocation
Coeff(2:VolNumber)=struct('Phix',zeros(length(lambda),nmax+1),...
    'Phisx',zeros(length(lambda),nmax+1),...
    'Chix',zeros(length(lambda),nmax+1),...
    'Chisx',zeros(length(lambda),nmax+1),...
    'PhixNp1',zeros(length(lambda),nmax),...
    'PhisxNp1',zeros(length(lambda),nmax),...
    'ChixNp1',zeros(length(lambda),nmax),...
    'ChisxNp1',zeros(length(lambda),nmax),...
    'Xix',zeros(length(lambda),nmax),...
    'Xisx',zeros(length(lambda),nmax),...
    'PhixPrime',zeros(length(lambda),nmax),...
    'PhisxPrime',zeros(length(lambda),nmax),...
    'ChixPrime',zeros(length(lambda),nmax),...
    'ChisxPrime',zeros(length(lambda),nmax),...
    'XixPrime',zeros(length(lambda),nmax),...
    'XisxPrime',zeros(length(lambda),nmax),...
    'NnGamma',zeros(length(lambda),nmax),...
    'DnGamma',zeros(length(lambda),nmax),...
    'NnDelta',zeros(length(lambda),nmax),...
    'DnDelta',zeros(length(lambda),nmax),...
    'Gamman',zeros(length(lambda),nmax),...
    'Deltan',zeros(length(lambda),nmax),...
    'NnA',0,...
    'DnA',zeros(length(lambda),nmax),...
    'NnB',0,...
    'DnB',zeros(length(lambda),nmax),...
    'An',zeros(length(lambda),nmax),...
    'Bn',zeros(length(lambda),nmax),...
    'alphan',zeros(length(lambda),nmax),...
    'betan',zeros(length(lambda),nmax),...
    'gamman',zeros(length(lambda),nmax),...
    'deltan',zeros(length(lambda),nmax));

for volume = 2:VolNumber
    n=1:nmax+1;
    
    Coeff(volume).Phix=repmat(((pi*x(:,volume)/2).^(1/2)), 1, nmax+1).*...
        besselj(repmat(n+1/2,length(x(:,volume)),1),repmat(x(:,volume),1,length(n))); %good
    Coeff(volume).Phisx=repmat(((pi*sx(:,volume)/2).^(1/2)), 1, nmax+1).*...
        besselj(repmat(n+1/2,length(sx(:,volume)),1),repmat(sx(:,volume),1,length(n))); %good
    Coeff(volume).Chix=repmat(((pi*x(:,volume)/2).^(1/2)), 1, nmax+1).*...
        bessely(repmat(n+1/2,length(x(:,volume)),1),repmat(x(:,volume),1,length(n))); %good
    Coeff(volume).Chisx=repmat(((pi*sx(:,volume)/2).^(1/2)), 1, nmax+1).*...
        bessely(repmat(n+1/2,length(sx(:,volume)),1),repmat(sx(:,volume),1,length(n))); %good
    
    Coeff(volume).PhixNp1=Coeff(volume).Phix(:,2:end);
    Coeff(volume).Phix=Coeff(volume).Phix(:,1:end-1);
    Coeff(volume).PhisxNp1=Coeff(volume).Phisx(:,2:end);
    Coeff(volume).Phisx=Coeff(volume).Phisx(:,1:end-1);
    Coeff(volume).ChixNp1=Coeff(volume).Chix(:,2:end);
    Coeff(volume).Chix=Coeff(volume).Chix(:,1:end-1);
    Coeff(volume).ChisxNp1=Coeff(volume).Chisx(:,2:end);
    Coeff(volume).Chisx=Coeff(volume).Chisx(:,1:end-1);
    
    Coeff(volume).Xix=Coeff(volume).Phix + 1i*Coeff(volume).Chix;
    Coeff(volume).Xisx=Coeff(volume).Phisx + 1i*Coeff(volume).Chisx;
    
    n=n(1:end-1);
    
    Coeff(volume).PhixPrime=((1./x(:,volume))*(n+1)).*...
        Coeff(volume).Phix-Coeff(volume).PhixNp1;
    Coeff(volume).PhisxPrime=((1./sx(:,volume))*(n+1)).*...
        Coeff(volume).Phisx-Coeff(volume).PhisxNp1;
    Coeff(volume).ChixPrime=((1./x(:,volume))*(n+1)).*...
        Coeff(volume).Chix-Coeff(volume).ChixNp1;
    Coeff(volume).ChisxPrime=((1./sx(:,volume))*(n+1)).*...
        Coeff(volume).Chisx-Coeff(volume).ChisxNp1;
    Coeff(volume).XixPrime=Coeff(volume).PhixPrime + 1i*...
        Coeff(volume).ChixPrime;
    Coeff(volume).XisxPrime=Coeff(volume).PhisxPrime + 1i*...
        Coeff(volume).ChisxPrime;
end

Coeff(1).Gamman = zeros(length(lambda),length(n));
Coeff(1).Deltan = zeros(length(lambda),length(n));

for volume = 2:VolNumber
    
    Coeff(volume).NnGamma = ((Coeff(volume).Phisx + ...
        Coeff(volume-1).Gamman.*Coeff(volume).Xisx).*...
        Coeff(volume).PhixPrime)-(repmat(s(:,volume),1,length(n)).*...
        Coeff(volume).Phix.*(Coeff(volume).PhisxPrime + ...
        Coeff(volume-1).Gamman.*Coeff(volume).XisxPrime));
    Coeff(volume).DnGamma = (repmat(s(:,volume),1,length(n)).*...
        Coeff(volume).Xix.*(Coeff(volume).PhisxPrime + ...
        Coeff(volume-1).Gamman.*Coeff(volume).XisxPrime))-...
        ((Coeff(volume).Phisx + Coeff(volume-1).Gamman.*...
        Coeff(volume).Xisx).*Coeff(volume).XixPrime);
    
    Coeff(volume).NnDelta = (Coeff(volume).Phix.*...
        (Coeff(volume).PhisxPrime + Coeff(volume-1).Deltan.*...
        Coeff(volume).XisxPrime))-(repmat(s(:,volume),1,length(n)).*...
        (Coeff(volume).Phisx + Coeff(volume-1).Deltan.*...
        Coeff(volume).Xisx).*Coeff(volume).PhixPrime);
    Coeff(volume).DnDelta = (repmat(s(:,volume),1,length(n)).*...
        (Coeff(volume).Phisx + Coeff(volume-1).Deltan.*...
        Coeff(volume).Xisx).*Coeff(volume).XixPrime)-...
        (Coeff(volume).Xix.*(Coeff(volume).PhisxPrime + ...
        Coeff(volume-1).Deltan.*Coeff(volume).XisxPrime));
    
    Coeff(volume).Gamman = Coeff(volume).NnGamma./Coeff(volume).DnGamma;
    Coeff(volume).Deltan = Coeff(volume).NnDelta./Coeff(volume).DnDelta;
    
    Coeff(volume).NnA = repmat(1i*s(:,volume),1,length(n));
    Coeff(volume).DnA = ((Coeff(volume).Phisx + Coeff(volume-1).Gamman.*...
        Coeff(volume).Xisx).*Coeff(volume).XixPrime) - ...
        (repmat(s(:,volume),1,length(n)).*Coeff(volume).Xix.*...
        (Coeff(volume).PhisxPrime + Coeff(volume-1).Gamman.*...
        Coeff(volume).XisxPrime));
    
    Coeff(volume).NnB = repmat(1i*s(:,volume),1,length(n));
    Coeff(volume).DnB = (repmat(s(:,volume),1,length(n)).*...
        (Coeff(volume).Phisx + Coeff(volume-1).Deltan.*...
        Coeff(volume).Xisx).*Coeff(volume).XixPrime)-(Coeff(volume).Xix.*...
        (Coeff(volume).PhisxPrime + Coeff(volume-1).Deltan.*...
        Coeff(volume).XisxPrime));
    
    Coeff(volume).An = Coeff(volume).NnA./Coeff(volume).DnA;
    Coeff(volume).Bn = Coeff(volume).NnB./Coeff(volume).DnB;
end

Coeff(VolNumber).alphan = repmat((1i.^(n+1)).*sqrt(pi*((2*n)+1)), ...
        length(lambda), 1);   % Plane wave excitation
Coeff(VolNumber).betan = repmat((1i.^(n+1)).*sqrt(pi*((2*n)+1)), ...
        length(lambda), 1);    % Plane wave excitation

for volume = VolNumber-1:-1:1
    Coeff(volume).alphan = Coeff(volume+1).An .* Coeff(volume+1).alphan;
    Coeff(volume).betan = Coeff(volume+1).Bn .* Coeff(volume+1).betan;
end

for volume = 1:VolNumber
    Coeff(volume).gamman = Coeff(volume).Gamman .* Coeff(volume).alphan;
    Coeff(volume).deltan = Coeff(volume).Deltan .* Coeff(volume).betan;
end




switch method
    case 1  % Calculation of absorption, scattering and absorption 
            % cross sections (or spectra if lambda has more than 1 element).
        sigmascan=(2*pi./(k(:,VolNumber).^2))*(2*n+1).*...
            ((abs(Coeff(VolNumber).Gamman)).^2 + ...
            (abs(Coeff(VolNumber).Deltan)).^2);
        sigmaextn=-(2*pi./(k(:,VolNumber).^2))*(2*n+1).*...
            (real(Coeff(VolNumber).Gamman) + real(Coeff(VolNumber).Deltan));
        
        sigmaabsn=sigmaextn-sigmascan;
        
        sigmasca=sum(sigmascan,2);
        sigmaext=sum(sigmaextn,2);
        sigmaabs=sigmaext-sigmasca;

    case 2  % E field in the xz plane (wave is propagating along z axis 
            % and polarized along x axis) with E0=1
        x=transpose(0:dx:maxx);
        z=-maxz:dz:maxz;
        xmatrix=x*ones(size(z));
        zmatrix=ones(size(x))*z;
        Rmatrix=sqrt((xmatrix.^2)+(zmatrix.^2));
        Thetamatrix=atan2(xmatrix,zmatrix);
        Thetamatrix(1,:)=Thetamatrix(1,:)-pi/10000;
        Thetamatrix(1,:)=abs(Thetamatrix(1,:));
        phi=0;

        kr=zeros(size(Rmatrix));
        alphan=zeros([size(Rmatrix) nmax]);
        betan=zeros([size(Rmatrix) nmax]);
        gamman=zeros([size(Rmatrix) nmax]);
        deltan=zeros([size(Rmatrix) nmax]);
        for volume = 1:VolNumber
            Rmask=(Rmatrix>=diameter(volume)/2).*(Rmatrix<diameter(volume+1)/2);
            kr = kr + k(volume)*Rmatrix.*Rmask;
            Rmask=repmat(Rmask,1,1,nmax);
            if volume<VolNumber
                alphan = alphan + repmat( reshape(Coeff(volume).alphan,[1,1,nmax]), size(Rmatrix)).*Rmask;
                betan = betan + repmat( reshape(Coeff(volume).betan,[1,1,nmax]), size(Rmatrix)).*Rmask;
            end
            gamman = gamman + repmat( reshape(Coeff(volume).gamman,[1,1,nmax]), size(Rmatrix)).*Rmask;
            deltan = deltan + repmat( reshape(Coeff(volume).deltan,[1,1,nmax]), size(Rmatrix)).*Rmask;
        end
        
        Eincr=sin(Thetamatrix).*exp(1i*kr.*cos(Thetamatrix))*cos(phi);
        EincTheta=cos(Thetamatrix).*exp(1i*kr.*cos(Thetamatrix))*cos(phi);
        
        Er=zeros(size(Rmatrix));
        ETheta=zeros(size(Rmatrix));
        
        for w=1:size(z,2)
            n=1:nmax+1;
            Z10nkr=(((pi./(2*kr(:,w))).^(1/2))*ones(size(n))).*...
                besselj(repmat(n+1/2,length(kr(:,w)),1),repmat(kr(:,w),1,length(n)));
            Z10nkrNp1=Z10nkr(:,2:end);
            Z10nkr=Z10nkr(:,1:end-1);
            n=n(1:end-1);
            Z11nkr=Z10nkr./(kr(:,w)*ones(size(n)));
            Z12nkr=((1./kr(:,w))*(n+1)).*Z10nkr-Z10nkrNp1;
            
            n=1:nmax+1;
            Z30nkr=(((pi./(2*kr(:,w))).^(1/2))*ones(size(n))).*...
                besselj(repmat(n+1/2,length(kr(:,w)),1),repmat(kr(:,w),1,length(n))) + ...
                1i*(((pi./(2*kr(:,w))).^(1/2))*ones(size(n))).*...
                bessely(repmat(n+1/2,length(kr(:,w)),1),repmat(kr(:,w),1,length(n)));
            Z30nkrNp1=Z30nkr(:,2:end);
            Z30nkr=Z30nkr(:,1:end-1);
            n=n(1:end-1);
            Z31nkr=Z30nkr./(kr(:,w)*ones(size(n)));
            Z32nkr=((1./kr(:,w))*(n+1)).*Z30nkr-Z30nkrNp1;
            
            m=1;
            Pn(1).Pndata = transpose(legendre(1,cos(Thetamatrix(:,w)))); % Vector of Legendre function for n=1
            PnmcosTheta(:,1)= Pn(1).Pndata(:,m+1); %Legendre function with m=1 for n=1
            Pnm1cosTheta(:,1)= zeros(size(Thetamatrix(:,w))); %Legendre function with m=2 for n=1, that is 0
            for n=2:nmax
                Pn(n).Pndata = transpose(legendre(n,cos(Thetamatrix(:,w)))); % Vector of Legendre function
                PnmcosTheta(:,n)= Pn(n).Pndata(:,m+1); % Legendre function with m=m
                Pnm1cosTheta(:,n)= Pn(n).Pndata(:,m+2); % Legendre function with m=2
            end
            n=1:nmax;
            TnmTheta = (ones(size(Thetamatrix(:,w))) * ...
                (sqrt(((2*n+1).*factorial(n-m))./(4*pi*(n.*(n+1)).*...
                factorial(n+m))))).*PnmcosTheta; %Possibilit de simplifier avec m=1
            T1nmTheta = m*TnmTheta./(sin(Thetamatrix(:,w)) * ones(size(n)));
            T2nmTheta = (ones(size(Thetamatrix(:,w)))*(n.*(n+1))).*TnmTheta;
            T3nmTheta = (ones(size(Thetamatrix(:,w)))*...
                sqrt(((2*n+1).*factorial(n-m))./(4*pi*(n.*(n+1)).*...
                factorial(n+m)))).*(((m*cos(Thetamatrix(:,w))./...
                sin(Thetamatrix(:,w)))*ones(size(n))).*PnmcosTheta + ...
                Pnm1cosTheta);
            
%             M1nmr = 0;
            M1nmTheta = 1i*Z10nkr.*T1nmTheta;
            
            N1nmr = Z11nkr.*T2nmTheta;
            N1nmTheta = Z12nkr.*T3nmTheta;
            
%             M3nmr = 0;
            M3nmTheta = 1i*Z30nkr.*T1nmTheta;
            
            N3nmr = Z31nkr.*T2nmTheta;
            N3nmTheta = Z32nkr.*T3nmTheta;
            
            ErVect = 2*squeeze(betan(:,w,:)).*N1nmr + ...
                2*squeeze(deltan(:,w,:)).*N3nmr;
            Er(:,w) = sum(ErVect,2);
            EThetaVect = 2*squeeze(alphan(:,w,:)).*M1nmTheta + ...
                2*squeeze(betan(:,w,:)).*N1nmTheta + ...
                2*squeeze(gamman(:,w,:)).*M3nmTheta + ...
                2*squeeze(deltan(:,w,:)).*N3nmTheta;
            ETheta(:,w) = sum(EThetaVect,2);
        end
        
        Er = Er + Eincr.*(Rmatrix>=diameter(VolNumber)/2);
        ETheta = ETheta + EincTheta.*(Rmatrix>=diameter(VolNumber)/2);
        
        ETot = sqrt((abs(Er).^2)+(abs(ETheta).^2));
        
        
    case 3  % E field on a shell of radius shell_rad nm around the particle 
            % (do NOT use negative value!)
            
        theta=transpose(0:dtheta:pi);
        theta(1)=pi/10000; %Prevent divide by zero
        theta(end)=pi*(1-1/10000); %Prevent divide by zero
        phi=0:dphi:pi/2;
        
        m=1;
        Pn = transpose(legendre(1,cos(theta))); %vector of Legendre
            % function for n=1
        PnmcosTheta(:,1)= Pn(:,m+1); %Legendre function with m=1 for n=1
        Pnm1cosTheta(:,1)= zeros(size(theta)); %Legendre function with m=2
            % for n=1, that is 0
        for n=2:nmax
            Pn = transpose(legendre(n,cos(theta))); %vector of
                % Legendre function
            PnmcosTheta(:,n)= Pn(:,m+1); %Legendre function with m=m
            Pnm1cosTheta(:,n)= Pn(:,m+2); %Legendre function with m=2
        end
        n=1:nmax;
        TnmTheta = (ones(size(theta)) * (sqrt(((2*n+1).*factorial(n-m))./...
            (4*pi*(n.*(n+1)).*factorial(n+m))))).*PnmcosTheta;
                %Possibilit de simplifier avec m=1
        T1nmTheta = m*TnmTheta./(sin(theta) * ones (size(n)));
        T2nmTheta = (ones(size(theta))*(n.*(n+1))).*TnmTheta;
        T3nmTheta = (ones(size(theta))*sqrt(((2*n+1).*factorial(n-m))./...
            (4*pi*(n.*(n+1)).*factorial(n+m)))).*...
            (((m*cos(theta)./sin(theta))*ones(size(n))).*PnmcosTheta + ...
            Pnm1cosTheta);
        
        ETot=zeros(length(theta),length(phi)); %Initialize the field matrix
        
        for l=1:length(shell_rad)
            index=find((diameter-2*shell_rad(l))>0);
            kM = 2*pi/(lambda/nindex(index(1)-1));
            alphan = Coeff(index(1)-1).alphan;
            betan = Coeff(index(1)-1).betan;
            gamman = Coeff(index(1)-1).gamman;
            deltan = Coeff(index(1)-1).deltan;
            
            kr=kM*(shell_rad(l));

            
            n=1:nmax + 1;
            Z10nkr=((pi/(2*kr))^(1/2))*besselj(n+1/2,kr);
            Z10nkrNp1=Z10nkr(2:end);
            Z10nkr=Z10nkr(1:end-1);
            n=n(1:end-1);
            Z11nkr=Z10nkr/kr;
            Z12nkr=((n+1)/kr).*Z10nkr-Z10nkrNp1;
            
            n=1:nmax + 1;
            Z30nkr=((pi/(2*kr))^(1/2))*besselj(n+1/2,kr) + ...
                1i*((pi/(2*kr))^(1/2))*bessely(n+1/2,kr);
            Z30nkrNp1=Z30nkr(2:end);
            Z30nkr=Z30nkr(1:end-1);
            n=n(1:end-1);
            Z31nkr=Z30nkr/kr;
            Z32nkr=((n+1)/kr).*Z30nkr-Z30nkrNp1;
            
%             M1nmr = 0;
            M1nmTheta = 1i*(ones(size(theta))*Z10nkr).*T1nmTheta;
            M1nmPhi = -(ones(size(theta))*Z10nkr).*T3nmTheta;
            
%             M3nmr = 0;
            M3nmTheta = 1i*(ones(size(theta))*Z30nkr).*T1nmTheta;
            M3nmPhi = -(ones(size(theta))*Z30nkr).*T3nmTheta;
            
            N1nmr = (ones(size(theta))*Z11nkr).*T2nmTheta;
            N1nmTheta = (ones(size(theta))*Z12nkr).*T3nmTheta;
            N1nmPhi = 1i*(ones(size(theta))*Z12nkr).*T1nmTheta;
            
            N3nmr = (ones(size(theta))*Z31nkr).*T2nmTheta;
            N3nmTheta = (ones(size(theta))*Z32nkr).*T3nmTheta;
            N3nmPhi = 1i*(ones(size(theta))*Z32nkr).*T1nmTheta;
        
            ErVect = 2*((ones(size(theta))*(betan)).*N1nmr + (ones(size(theta))*(deltan)).*N3nmr); %(cosPhi)
            ErPhi0 = sum(ErVect,2);
            Er = ErPhi0*cos(phi);
            
            EThetaVect = 2* ((ones(size(theta))*(alphan)).*M1nmTheta + (ones(size(theta))*(betan)).*N1nmTheta + ...
                (ones(size(theta))*(gamman)).*M3nmTheta + (ones(size(theta))*(deltan)).*N3nmTheta); %(cosPhi)
            EThetaPhi0 = sum(EThetaVect,2);
            ETheta = EThetaPhi0*cos(phi);
            
            EPhiVect = 2* ((ones(size(theta))*(alphan)).*M1nmPhi + (ones(size(theta))*(betan)).*N1nmPhi + ...
                (ones(size(theta))*(gamman)).*M3nmPhi + (ones(size(theta))*(deltan)).*N3nmPhi); %(sinPhi)
            EPhiPhi0 = sum(EPhiVect,2);
            EPhi = EPhiPhi0*sin(phi);
            
            ETot(:,:,l)=sqrt((abs(Er).^2)+(abs(ETheta).^2)+(abs(EPhi).^2));
        end
        
    case 4  %Scattered field in the far-field

        gamman = Coeff(end).gamman;
        deltan = Coeff(end).deltan;

        theta=transpose(0:dtheta:pi);
        theta(1)=pi/10000; %Prevent divide by zero
        theta(end)=pi*(1-1/10000); %Prevent divide by zero
        phi=0:dphi:pi/2;
        
        m=1;
        Pn = transpose(legendre(1,cos(theta))); %vector of Legendre
            % function for n=1
        PnmcosTheta(:,1)= Pn(:,m+1); %Legendre function with m=1 for n=1
        Pnm1cosTheta(:,1)= zeros(size(theta)); %Legendre function with m=2
            % for n=1, that is 0
        for n=2:nmax
            Pn = transpose(legendre(n,cos(theta))); %vector of
                % Legendre function
            PnmcosTheta(:,n)= Pn(:,m+1); %Legendre function with m=m
            Pnm1cosTheta(:,n)= Pn(:,m+2); %Legendre function with m=2
        end
        n=1:nmax;
        TnmTheta = (ones(size(theta)) * (sqrt(((2*n+1).*factorial(n-m))./...
            (4*pi*(n.*(n+1)).*factorial(n+m))))).*PnmcosTheta;
                %Possibilit de simplifier avec m=1
        T1nmTheta = m*TnmTheta./(sin(theta) * ones (size(n)));
%         T2nmTheta = (ones(size(theta))*(n.*(n+1))).*TnmTheta; % Unused
        T3nmTheta = (ones(size(theta))*sqrt(((2*n+1).*factorial(n-m))./...
            (4*pi*(n.*(n+1)).*factorial(n+m)))).*...
            (((m*cos(theta)./sin(theta))*ones(size(n))).*PnmcosTheta + ...
            Pnm1cosTheta);
        
%         M3nmr = 0;
        M3nmTheta = 1i*(ones(size(theta))*(-1i).^(n+1)).*T1nmTheta;
        M3nmPhi = -(ones(size(theta))*(-1i).^(n+1)).*T3nmTheta;
        
%         N3nmr = 0;
        N3nmTheta = (ones(size(theta))*(-1i).^n).*T3nmTheta;
        N3nmPhi = 1i*(ones(size(theta))*(-1i).^n).*T1nmTheta;
        
%         Escar = 0;
        EscaThetaVect = (ones(size(theta))*2*(gamman)).*M3nmTheta + ...
            (ones(size(theta))*2*(deltan)).*N3nmTheta; %(cosPhi)
        EscaThetaPhi0 = sum(EscaThetaVect,2);
        EscaTheta = EscaThetaPhi0*cos(phi);
        EscaPhiVect = (ones(size(theta))*2*(gamman)).*M3nmPhi + ...
            (ones(size(theta))*2*(deltan)).*N3nmPhi; %(sinPhi)
        EscaPhiPhi0 = sum(EscaPhiVect,2);
        EscaPhi = EscaPhiPhi0*sin(phi);
        
        
        EscaTot=sqrt((abs(EscaTheta).^2)+(abs(EscaPhi).^2));
        
    otherwise
        error('Unexpected value of method input')
end


switch method
    case 1 % Calcul de Cabs, Csca et Cext
        if separaten==1
            varargout{1} = sigmaabsn;
            varargout{2} = sigmascan;
            varargout{3} = sigmaextn;
        else
            varargout{1} = sigmaabs;
            varargout{2} = sigmasca;
            varargout{3} = sigmaext;
        end
        
    case 2 % Calcul du champ selon une coupe XZ
        varargout{1} = ETot;
        
    case 3 % Calcul du champ en surface
        varargout{1} = ETot;
        
    case 4 % Calcul du champ diffus en far field
        varargout{1} = EscaTot;
end